 
#    Copyright 2021 KeinShin
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#        http://www.apache.org/licenses/LICENSE-2.0
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import re
from requests.sessions import session
from requests_html import HTMLSession
from bs4 import BeautifulSoup
import requests
from collections import OrderedDict



from  .crawl import crawl
class channel:
    def __init__(self,channel_id:str='',channellink:str="",channelname:str=''):
        self.session = HTMLSession()
        if channel_id:
            self.channel="https://www.youtube.com/channel/" + channel_id
        elif channellink:
            self.channel=channellink
        elif channelname:
            response = self.session.get("https://www.youtube.com/results?search_query=" + channelname)
            response.html.render(sleep=1)
            
            soup = BeautifulSoup(response.html.html, "html.parser") 
            self.channel=soup.find("a",{"class":"channel-link yt-simple-endpoint style-scope ytd-channel-renderer"}).get("href")
            self.id=self.channel
            self.channel="https://www.youtube.com" + self.channel
        else:
            raise ValueError("No Paramter is provided")
        response = self.session.get(self.channel)
        response.html.render(sleep=1)
        self.soup = BeautifulSoup(response.html.html, "html.parser") 
        
        
    def subs(self):
            return self.soup.find("yt-formatted-string",{"id":"subscriber-count"}).text
     
    
    def about(self):
        response = self.session.get('https://www.youtube.com' + self.id + "/" + "about")
        response.html.render(sleep=1)
        soup = BeautifulSoup(response.html.html, "html.parser")  
        return soup

    def description(self):
        self.chan=self.about()
        return self.chan.find("yt-formatted-string",{"id":"description"}).text 
    
    
    def joined(self):
        about=self.about()
        j=[]
        for i in about.find_all("span",{"class":"style-scope yt-formatted-string"}):
        
            j.append(i.get_text())
        jp=int(j.index("Joined "))
        return j[jp +1]
    
    
    def links(self):
        print("Getting Links of about Please wait!!!")
        print("Processing Request")
        lip=self.about()
        li={}
        laps=[]
        gaps=[]
        user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7'
        headers = {'User-Agent':user_agent} 
        for p in lip.find_all("yt-formatted-string",{"class":"info-text style-scope ytd-channel-about-metadata-renderer"}):
            for x in lip.find_all("a",{"class":"yt-simple-endpoint style-scope ytd-channel-about-metadata-renderer"}):
                x=x.get('href')
                if x.find("redirect") != -1:
                 
                 session=HTMLSession()
                 response = session.get(x)
                 response.html.render(sleep=1)
                 x = BeautifulSoup(response.html.html, "html.parser")  
                 x=x.find_all("div",attrs={"id":"redirect-action-container"})
                 for i in x:
                     x=i.find("a")['href']
                laps.append(x)
                    
                gaps.append(p.text)
        gaps=list(OrderedDict.fromkeys(gaps))
        laps=list(OrderedDict.fromkeys(laps))
        for i,p in zip(gaps,laps):
            li.update({i:p})
        
        return li

    def latest_video(self):
     print('https://www.youtube.com' + self.id + "/" + "videos")
     response = self.session.get('https://www.youtube.com' + self.id + "/" + "videos")
     response.html.render(sleep=1)
     soup = BeautifulSoup(response.html.html, "html.parser")  
     op=soup.find("a",{"id":"thumbnail"}).get('href')
     op=crawl(video_link="https://www.youtube.com" + op)
     return {"Latest Video" : op.VideoDetails()}
    
    def latest_community(self):
     response = self.session.get('https://www.youtube.com' + self.id + "/" + "community")
     response.html.render(sleep=1)
     soup = BeautifulSoup(response.html.html, "html.parser")  
     comun=soup.find("yt-formatted-string",{"id":"content-text"}).text
     return comun
 
    def spareChannels(self):
      session=HTMLSession()
      response = session.get('https://www.youtube.com' + self.id + "/" + "channels")
      response.html.render(sleep=1)
      soup = BeautifulSoup(response.html.html, "html.parser")
      channels={}
      for i in soup.find_all("div",{"id":"channel"}):
           response = session.get('https://www.youtube.com' + i.find("a")['href'])
           response.html.render(sleep=1)
           soup = BeautifulSoup(response.html.html, "html.parser")
           channels.update({i.find('span').text:{"Link":'https://www.youtube.com' + i.find("a")['href'],"Subscribers":soup.find("yt-formatted-string",{"id":"subscriber-count"}).text }})
      return channels



  
#    Copyright 2021 KeinShin
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#        http://www.apache.org/licenses/LICENSE-2.0
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from requests_html import AsyncHTMLSession
from bs4 import BeautifulSoup
class Init:
        async def init(self,link):
         self.link2=link
        
         session = AsyncHTMLSession()
         response = await session.get(self.link2)
         await response.html.arender(sleep=1,timeout=30)
         return BeautifulSoup(response.html.html, "html.parser") 
    



from requests_html import AsyncHTMLSession
from bs4 import BeautifulSoup

class Init:
        async def init(self,link):
         self.link2=link
        
         session = AsyncHTMLSession()
         response = await session.get(self.link2)
         await response.html.arender(sleep=1,timeout=30)
         return BeautifulSoup(response.html.html, "html.parser") 
     


#    Copyright 2021 KeinShin
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#        http://www.apache.org/licenses/LICENSE-2.0
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from bs4 import BeautifulSoup
import requests
import moviepy.editor  as  mp
from requests_html import HTMLSession 
import os
import urllib.request
import re
from urllib.request import urlopen,Request
from lxml import etree
from requests.api import head
import pytube  
from pytube import YouTube  


class crawl:
    
    def __init__(self,video_name:str="",video_id:str="",output: str = "",video_link:str=''):
        self.name = video_name
        
        self.id = video_id
        self.out= output
        self.link = video_link
        if not self.name:
            if not self.id:
              if not self.link:
                raise ValueError("Provide Video Name Or Video Link or video id, Please.")
        if self.link:
            self.link=self.link
        elif self.id:
            self.link="https://www.youtube.com/watch?v=" + self.id
        elif self.name:        
            html = urllib.request.urlopen("https://www.youtube.com/results?search_query="+ self.name)
            video_ids = re.findall(r"watch\?v=(\S{11})", html.read().decode())
            self.vidid=video_ids[0]
            self.link="https://www.youtube.com/watch?v=" + self.vidid
        self.link2=self.link
        user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7'
        headers = {'User-Agent':user_agent} 
        self.link=self.link2
    
    
        session = HTMLSession()
        response = session.get(self.link2)
        response.html.render(sleep=1)
        self.soup = BeautifulSoup(response.html.html, "html.parser") 
    
    
    
    
    
    
    def keyword(self):
            
        self.title = self.soup.find("meta",attrs={"name":"keywords"})
        
        return self.title["content"]
    
       
    def videolink(self):
        return self.vidid
    
    def VideoDetails(self):

        likes=self.likes_dislikes()[1]
        dislikes=self.likes_dislikes()[2]
        if dislikes == "Dislike":
            dislikes= "N/A"
        title=self.upload_time_and_title()[0]
        upload=self.upload_time_and_title()[1]
        params = {
         "Video Title": title,
         "Video Id" : self.videolink().split("/")[-1], 
         "Description": self.description(),
         "Veiws": self.veiws(),
         "Likes": likes,
         "Dislikes": dislikes,
         "Upload Time": upload,
         "Video Link":self.videolink(),
         "Channel" : self.channel()
        }
        return params
    
    
    def likes_dislikes(self):
        session = HTMLSession()
        response = session.get(self.link2)
        response.html.render(sleep=1)
        soup = BeautifulSoup(response.html.html, "html.parser")
        likes_dislikes=[]
        for i in soup.find_all("yt-formatted-string",{"id":"text"}):
            likes_dislikes.append(i.get_text())
        return likes_dislikes
    
    
    def upload_time_and_title(self):
        det=[]
        title=self.soup.find_all("yt-formatted-string",{"class":"style-scope ytd-video-primary-info-renderer"})
        for i in title:
            det.append(i.get_text())
        return det
    
    def VidTitle(self):
        return self.upload_time_and_title()[0]

    
    def videoUploadTime(self):
        return self.upload_time_and_title()[1]
        

    
    def description(self):
        descrip=''
        for i in self.soup.find_all("span",{"class":"style-scope yt-formatted-string"}):
            descrip+=i.get_text()
        return descrip
    
    
    def channel(self):
        channelinfo={}
        name=self.soup.find("yt-formatted-string", {"class": "ytd-channel-name"})
        tg=name
        name=name.find("a").text
        channelink=self.soup.find("a",{"class":"yt-simple-endpoint style-scope ytd-video-owner-renderer"})
        channelink=channelink.get("href")
        subs=self.soup.find("yt-formatted-string",{"id":"owner-sub-count"})
        subs=subs.get_text()
        channelinfo.update({"name":name,"channel_link":"https://www.youtube.com"+ channelink,
        "subscriber": subs                
        })
        return channelinfo
    
    
    def download(self,to_mp3:bool=False,mp3name:str=''):
        
        if not self.out:
            path='.'
        else:
            path=self.out
        yt = YouTube(self.link2)
        yt = yt.streams.filter(progressive=True, file_extension='mp4').order_by('resolution').desc().first()

        yt.download(path)
                
    
    def veiws(self):
        veiws=self.soup.find("span",{"class":"view-count style-scope ytd-video-view-count-renderer"})
        return veiws.text
    
    
    def videolink(self):
      ida=self.soup.find("link",{"rel":"shortlinkUrl"})
      return ida.get("href")




from setuptools import setup, find_packages
import codecs
import os

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
    long_description = long_description.replace("\r","") # Do not forget this line
except OSError:
    print("Pandoc not found. Long_description conversion failure.")
    import io
    # pandoc is not installed, fallback to using raw contents
    with io.open('README.md', encoding="utf-8") as f:
        long_description = f.read()

VERSION = '0.17'
DESCRIPTION ="Youtube videos and channels scraper python wraper!"
LONG_DESCRIPTION = 'A package that allows you to scrap data from youtube videos and channel.'

# Setting up
setup(
    name="youtubecrawler",
    version=VERSION,
    author="KeinShin",
    author_email="<youtubecrawler@youtubecrawl.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=['youtubecrawler','youtubecrawler.Asyncs'],
    install_requires=['moviepy', 'requests_html', 'bs4','lxml','pytube'],
    keywords=['python', 'youtube', 'scraper', 'youtubevideodownload', 'channelscraper', 'youtubescraper'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


  
#    Copyright 2021 KeinShin
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#        http://www.apache.org/licenses/LICENSE-2.0
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from . import Init

import re
from requests.sessions import session
from requests_html import HTMLSession
from bs4 import BeautifulSoup
import requests
from collections import OrderedDict

from .asynced import crawl as Crawl
class channel(Init):
    def __init__(self,**args):
      channel_id=args.get("channel_id")
      channellink=args.get('channel_link')
      channelname=args.get("channelname")
      self.channel_id=channel_id
      self.channellink=channellink
      self.channelname=channelname
    async def Ainit(self): 
      
        if self.channel_id:
            self.channel="https://www.youtube.com/channel/" + self.channel_id
            self.chanid=self.channel_id
        elif self.channellink:
       
            self.channel=self.channellink
            c=Crawl(video_link=self.channel)
            self.chanid=(await c.videolink()).split("/")[-1]
        elif self.channelname:
            soup =await  self.init("https://www.youtube.com/results?search_query=" + self.channelname)
            self.channel=soup.find("a",{"class":"channel-link yt-simple-endpoint style-scope ytd-channel-renderer"}).get("href")
            self.chanid=self.channel
            
            self.channel="https://www.youtube.com" + self.channel
        else:
            raise ValueError("No Paramter is provided")
        self.chanid=self.chanid
        
    async def subs(self):
            await self.Ainit()
            self.soup=await self.init(self.channel)
            return self.soup.find("yt-formatted-string",{"id":"subscriber-count"}).text
     
    
    async def about(self):
       await self.Ainit()

       return await self.init('https://www.youtube.com' + self.chanid + "/" + "about")

    async def description(self):
        await self.Ainit()
        self.chan= await self.about()
        return self.chan.find("yt-formatted-string",{"id":"description"}).text 
    
    
    async def joined(self):
        await self.Ainit()
        about=await self.about()
        j=[]
        for i in about.find_all("span",{"class":"style-scope yt-formatted-string"}):
        
            j.append(i.get_text())
        jp=int(j.index("Joined "))
        return j[jp +1]
    
    
    async def links(self):
        await self.Ainit()
        print("Getting Links of about Please wait!!!")
        print("Processing Request")
        lip=await self.about()
        li={}
        laps=[]
        gaps=[]
        user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7'
        headers = {'User-Agent':user_agent} 
        for p in lip.find_all("yt-formatted-string",{"class":"info-text style-scope ytd-channel-about-metadata-renderer"}):
            for x in lip.find_all("a",{"class":"yt-simple-endpoint style-scope ytd-channel-about-metadata-renderer"}):
                x=x.get('href')
                if x.find("redirect") != -1:
                 
                 x = self.init(x)
                 x=x.find_all("div",attrs={"id":"redirect-action-container"})
                 for i in x:
                     x=i.find("a")['href']
                laps.append(x)
                    
                gaps.append(p.text)
        gaps=list(OrderedDict.fromkeys(gaps))
        laps=list(OrderedDict.fromkeys(laps))
        for i,p in zip(gaps,laps):
            li.update({i:p})
        
        return li

    async def latest_video(self):
     await self.Ainit()
     soup =await self.init('https://www.youtube.com' + self.chanid + "/" + "videos")
     op=soup.find("a",{"id":"thumbnail"}).get('href')
     op=Crawl(video_link="https://www.youtube.com" + op)
     return {"Latest Video" : await op.VideoDetails()}
    
    async def latest_community(self):
     await self.Ainit()
     soup = await self.init('https://www.youtube.com' + self.chanid + "/" + "community")
     comun=soup.find("yt-formatted-string",{"id":"content-text"}).text
     return comun
 
    async def spareChannels(self):
      await self.Ainit()
      soup=await self.init(self.channel)

      channels={}
      for i in soup.find_all("div",{"id":"channel"}):
           soup = await self.init('https://www.youtube.com' + i.find("a")['href'])
           channels.update({i.find('span').text:{"Link":'https://www.youtube.com' + i.find("a")['href'],"Subscribers":soup.find("yt-formatted-string",{"id":"subscriber-count"}).text }})
      return channels
